
/* Chrysalide - Outil d'analyse de fichiers binaires
 * expression.c - gestion des expressions décompilées
 *
 * Copyright (C) 2010-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "expression.h"


#include "expression-int.h"



/* Initialise la classe des expressions décompilées. */
static void g_dec_expression_class_init(GDecExpressionClass *);

/* Initialise une instance d'expression décompilée. */
static void g_dec_expression_init(GDecExpression *);

/* Imprime pour l'écran la négation d'une expression décompilée. */
static GBufferLine *g_dec_expression_neg_print(const GDecExpression *, GCodeBuffer *, GBufferLine *, GLangOutput *);



/* Indique le type défini pour une expression décompilée. */
G_DEFINE_TYPE(GDecExpression, g_dec_expression, G_TYPE_DEC_INSTRUCTION);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des expressions décompilées.            *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dec_expression_class_init(GDecExpressionClass *klass)
{

}


/******************************************************************************
*                                                                             *
*  Paramètres  : expr = instance à initialiser.                               *
*                                                                             *
*  Description : Initialise une instance d'expression décompilée.             *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dec_expression_init(GDecExpression *expr)
{

}


/******************************************************************************
*                                                                             *
*  Paramètres  : expr   = expression à transcrire en version humaine.         *
*                buffer = tampon où doit se réaliser l'insertion.             *
*                line   = ligne d'impression prête à emploi ou NULL.          *
*                output = langage de programmation de sortie.                 *
*                                                                             *
*  Description : Imprime pour l'écran la négation d'une expression décompilée.*
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static GBufferLine *g_dec_expression_neg_print(const GDecExpression *expr, GCodeBuffer *buffer, GBufferLine *line, GLangOutput *output)
{
    GBufferLine *result;                    /* Ligne à retourner           */

    g_buffer_line_append_text(line, BLC_ASSEMBLY_HEAD, "!", 1, RTT_KEY_WORD, NULL);

    result = expr->original_print(G_DEC_INSTRUCTION(expr), buffer, line, output);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : expr = instance à traiter.                                   *
*                                                                             *
*  Description : Réalise une négation sur une expression décompilée.          *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void g_dec_expression_negate(GDecExpression *expr)
{
    bool status;                            /* Réussite de l'opération ?   */

    if (expr->negate != NULL)
        status = expr->negate(expr);
    else
        status = false;

    if (!status)
    {
        expr->original_print = G_DEC_INSTRUCTION(expr)->print;
        G_DEC_INSTRUCTION(expr)->print = (dec_instr_print_fc)g_dec_expression_neg_print;
    }

}
